/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/
//avr headers
#include <avr/io.h>
#include <string.h>
#include <inttypes.h>
#include <avr/interrupt.h>
#include <util/delay.h>
#include <avr/pgmspace.h>
//our headers
#include "macros.h"
#include "max7456_spi.h"
#include "max7456_learn.h"
#include "lab5.h"
#include "human_tetris.h"
#include "brick_breaker.h"
#include "whack_a_mole.h"
//uart debugging
#include <avr/io.h>
#include <stdio.h>
#include "uart.h"
// UART file descriptor
// putchar and getchar are in uart.c
FILE uart_str = FDEV_SETUP_STREAM(uart_putchar, uart_getchar, _FDEV_SETUP_RW);

// ========================== "PRIVATE" FUNCTION DECLARATIONS ==========================
void init_SPI(void);
void init_MISC(void);
void init_ADC(void);
void init_sync(void);
void init_ADC2(void);
void init_Buttons(void);

void debugDump();

void filterInput();
void display();
void debounce(uint8_t, uint8_t *, uint8_t *);

void GoMenu();
void GoStateMachine(void);

// ============================== "PRIVATE" VARIABLES ==============================

// ---- ADC Stuff ----
volatile uint8_t ADC_input;

#define ADC_PIN PINC
#define DDR_ADC_DATA DDRC

#define ADC_CLK PD7
#define DDR_ADC_CLK DDRD
#define ADC_CLK_PORT PORTD

// ---- NTSC Reading Stuff ----

#define LineHeight 6
#define LineShift 4 //tuned for pixel matching and fitting all lines
#define LastLine 254
#define FirstLine 20

volatile uint16_t lineNumber;
uint8_t sampledLineNumber;
uint8_t sampleNumber;
uint8_t jFast;
uint8_t read;
volatile uint8_t doneReading;

#define HSYNC_DDR DDRB
#define HSYNC_DDR_PIN DDB2
#define HSYNC_PIN PB2
#define HSYNC_ISR INT2
#define HSYNC_ISR_vect INT2_vect
#define HSYNC_ISC0 ISC20
#define HSYNC_ISC1 ISC21

#define VSYNC_DDR DDRD
#define VSYNC_DDR_PIN DDD3
#define VSYNC_PIN PD3
#define VSYNC_ISR INT1
#define VSYNC_ISR_vect INT1_vect
#define VSYNC_ISC0 ISC10
#define VSYNC_ISC1 ISC11

// ---- Event Counters ----

volatile int8_t time1 = 0;
int16_t readCounter = 0, buttonCounter = 0, heartBeatCounter = 0;

volatile int16_t lastGameTimeMs = 0;
volatile uint8_t frames = 0;

//State machine state names
#define NoPush 1
#define MaybePush 2
#define Pushed 3
#define MaybeNoPush 4

// ---- Output Stuff ----


#define LED_DDR DDRD
#define LED_DDR_PIN DDD2
#define LED_PORT PORTD
#define LED_PIN PD2

#define SW_DDR DDRA
#define SW_PIN PINA

#define SW1_TOGGLE PA4 //toggle
#define SW1_DDR_PIN DDA4
#define SW2_ENTER PA5 //enter
#define SW2_DDR_PIN DDA5

// ============================== ISRS ==============================

//gives timebase
ISR (TIMER1_COMPA_vect)
{
	if (++time1 >= 1) {
		gameTimeMs += 5;
		time1 = 0;
	}

}

volatile uint8_t vsyncs = 0;
//triggers on vertical sync pulse
ISR (VSYNC_ISR_vect)
{
	//only read at most every other frame and when other processing has asked for a frame
	if (doneReading == 0 && ((vsyncs++ & 0x01) == 0)) {
		lineNumber = 0;
		sampledLineNumber = 0;
		read = 0;
		SET(EIMSK,HSYNC_ISR); // turn on HSYNC triggering
	}
}

//triggers on horizontal sync pulse
ISR(HSYNC_ISR_vect)
{
	if (read) {
		//eat back porch porch - 5.9us (minus time to get sync/enter isr)
		_delay_us(2); //20nops per us, + loop setup time
		asm(
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t" // 15 and 4 us delay for 1.25MHz ADC CLK
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t"
				"NOP \n\t" // 19 and 2 us delay for 5 MHz ADC CLK (3 sample delay)
		);

		//sample this line into the row
		for (jFast = 0; jFast < COLS; jFast++) {
			Input_Data[sampledLineNumber][jFast] = ADC_PIN; //read this pixel
			//should be 0.8583us in between samples, 0.05us per cycle, so 17 cycles is closest
			//more nops here means a longer "line" is read in
			asm(
					"NOP \n\t"
					"NOP \n\t"
					"NOP \n\t"
					"NOP \n\t"
					"NOP \n\t" //should have 5 nops here optimally
			);
		}

		sampledLineNumber++;
	}

	// 34 lines of blank after vert sync pulse which took up 6 lines
	// then 485 img lines  = 525 total :-)
	//vertically downsample by LineHeight
	//look at 239 of 242 img lines to get 39

	read = lineNumber >= FirstLine && lineNumber < LastLine && ((lineNumber + LineShift) % LineHeight == 0);
	if (lineNumber == LastLine) {
		CLR(EIMSK,HSYNC_ISR); //turn off this isr, done reading frame
		frames++;
		doneReading = 1;
	}
	lineNumber++;
}

// ============================== FUNCTION BODIES ==============================

int main() {

	// init the UART -- uart_init() is in uart.c
	//	uart_init();
	//	stdout = stdin = stderr = &uart_str;
	//	fprintf(stdout, "Starting main...\n\r");

	init_SPI();

	//Pushing NEW chars to the MAX7456
#if WRITEALLCHARS
	learn_all_smart();
#endif

#if	WRITEALLCHARS | ALLCHARSDEBUG
	write_all_chars();
	while(1);
#endif

	//quickly learn 8 new chars
#if WRITECHARS
	learnCharacters(0);
	init_SPI();
#endif

	init_ADC();

	init_MISC();

	init_sync();

	init_ADC2();

	init_Buttons();

	GameState = MENU;
	gameTimeMs = 0;

	uint8_t a, b, c, d;

	enterPushed = &a;
	togglePushed = &b;
	enterFlag = &c;
	toggleFlag = &d;

	*enterPushed = NoPush;
	*togglePushed = NoPush;
	*enterFlag = 0;
	*toggleFlag = 0;

	//crank up the ISRs
	sei();

	SET(EIMSK,VSYNC_ISR); //enable VSYNC - ISR

	//printf("Hello World!\n\r");
	//debugging reads
	//	for (uint8_t i = 0x80; i < 0x9F; i++) {
	//		printf("reg %X is %X\n\r", i, spi_read_byte(i));
	//	}
	//	printf("STAT reg %X is %X\n\r", 0xA, spi_read_byte(0xA)); //STAT
	//	printf("OSDBL reg %X is %X\n\r", 0xEC, spi_read_byte(0xEC)); //OSDBL
	//	printf("VM0 reg %X is %X\n\r", 0x80, spi_read_byte(0x80)); //VM0

	while (1) {
		//if done reading, use this frame for everything else
		if (doneReading == 1) {

			//toggle LED ON and OFF each second
			if (heartBeatCounter == 0) {
				CLR(LED_PORT,LED_PIN);
			}
			if (heartBeatCounter == (mps / 2)) {
				SET(LED_PORT,LED_PIN);
			}

			//heart beat at 1Hz and read ADC
			if (++heartBeatCounter == mps) {
				heartBeatCounter = 0;

				// reading channel A.0 thru ADC
				Black_Level = ADCH;

				//start a new conversion
				SET(ADCSRA,ADSC);
			}

			//button state at around 30 Hz
			if (++buttonCounter == (mps / 30)) {
				buttonCounter = 0;
				debounce((~(SW_PIN) >> SW2_ENTER) & 0x01, enterPushed, enterFlag);
				debounce((~(SW_PIN) >> SW1_TOGGLE) & 0x01, togglePushed, toggleFlag);
			}

			//input/output game at adjustable fps, tracking mode runs 2x as fast
			++readCounter;
			if ((GameState == TRACKING && readCounter >= 1) || (readCounter >= 2)) {
				readCounter = 0;

				//filterInput(); //not needed

				//THIS DOES NOT PLAY WELL WITH THE GAME
				//if ((~(SW_PIN) >> SW2_ENTER) & 0x01) {
				//	debugDump();
				//}


				GoStateMachine(); //launch game state machine


				//				// display corners for calibration
				//				// col then row index
				//				loadString(1, 1, "A\0");
				//				loadString(1, 11, "B\0");
				//				loadString(27, 1, "C\0");
				//				loadString(27, 11, "D\0");
				display();
			}
			doneReading = 0; //kick off a new read
		} //end if(doneReading == 1)
	} //end while
}//end main

void PrintMenu(char a, char b, char c, char d) {
	sprintf(buffer, " Play Tetris ");
	loadFancyString(8, 4, buffer, a);
	sprintf(buffer, " Play Brick Breaker ");
	loadFancyString(5, 5, buffer, b);
	sprintf(buffer, " Play  Whack-A-Mole ");
	loadFancyString(5, 6, buffer, c);
	sprintf(buffer, " Tracking Demo ");
	loadFancyString(7, 8, buffer, d);

}

char warning[9][28] PROGMEM = { { "WARNING: This game may" }, { "  cause photo-sensitive" },
								 { "  seizures. Also extreme" }, { "  play motions may harm" },
								 { "  yourself or others or" }, { "  damage property. Consult" },
								 { "  your doctor before use." }, { "  Also your mom if" }, { "  applicable." } };

char zzz = 0;
void GoMenu() {
	//safety msg
	if (zzz <= 120) {
		zzz++;

		for (uint8_t i = 0; i < 9; i++)
			loadStringPrgm(2, i + 2, warning[i]);

		//		sprintf(buffer, "WARNING: This game may");
		//		loadString(2, 3, buffer);
		//		sprintf(buffer, "  cause photo-sensitive");
		//		loadString(2, 4, buffer);
		//		sprintf(buffer, "  seizures. Also extreme");
		//		loadString(2, 5, buffer);
		//		sprintf(buffer, "  play motions may harm");
		//		loadString(2, 6, buffer);
		//		sprintf(buffer, "  yourself or others or");
		//		loadString(2, 7, buffer);
		//		sprintf(buffer, "  damage property. Consult");
		//		loadString(2, 8, buffer);
		//		sprintf(buffer, "  your doctor before use.");
		//		loadString(2, 9, buffer);
		//		sprintf(buffer, "  Also your mom if");
		//		loadString(2, 10, buffer);
		//		sprintf(buffer, "  applicable.");
		//		loadString(2, 11, buffer);

		return;
	}

	sprintf(buffer, "ECE 4760: Final Project");
	loadString(3, 1, buffer);
	sprintf(buffer, "HUMAN  TETRIS");
	loadString(8, 2, buffer);

	sprintf(buffer, "Adam Papamarcos  (aip23)");
	loadString(3, 10, buffer);
	sprintf(buffer, "Kerran Flanagan  (kaf42)");
	loadString(3, 11, buffer);

	switch (ActiveMenuButton) {

	case PlayTetris:
		PrintMenu(LBC, 0, 0, 0);
		if (*enterFlag) {
			*enterFlag = 0;

			GameState = PLAYING_T;
			initTetris();
		}
		if (*toggleFlag) {
			ActiveMenuButton = PlayBrickBreaker;
			*toggleFlag = 0;
		}
		break;

	case PlayBrickBreaker:
		PrintMenu(0, LBC, 0, 0);
		if (*enterFlag) {
			*enterFlag = 0;
			GameState = PLAYING_BB;
			initBrickBreaker();
		}
		if (*toggleFlag) {
			ActiveMenuButton = PlayMole;
			*toggleFlag = 0;
		}
		break;

	case PlayMole:
		PrintMenu(0, 0, LBC, 0);
		if (*enterFlag) {
			*enterFlag = 0;

			GameState = PLAYING_MOLE;
			initMole();
		}
		if (*toggleFlag) {
			ActiveMenuButton = DoTracking;
			*toggleFlag = 0;
		}
		break;

	case DoTracking:
		PrintMenu(0, 0, 0, LBC);
		if (*enterFlag) {
			*enterFlag = 0;

			GameState = TRACKING;
			gameTimeMs = 0;
			lastGameTimeMs = 0;
			clearOutput();
		}
		if (*toggleFlag) {
			ActiveMenuButton = PlayTetris;
			*toggleFlag = 0;
		}
		break;

	default:

		break;
	}
}

void GoStateMachine() {
	switch (GameState) {
	case MENU:
		clearOutput();
		GoMenu();
		break;

	case PLAYING_T:
		DoTetrisFrame();

		if (*enterFlag) {
			GameState = MENU;

			*enterFlag = 0;
		}
		break;

	case PLAYING_BB:
		DoBrickBreakerFrame();

		if (*enterFlag) {
			GameState = MENU;

			*enterFlag = 0;
		}
		break;

	case PLAYING_MOLE:
		DoMoleFrame();

		if (*enterFlag) {
			GameState = MENU;

			*enterFlag = 0;
		}
		break;

	case GAMEOVER:
		sprintf(buffer, "GAME  OVER");
		loadFancyString(9, 6, buffer, INVERT | BLINK);

		if (*enterFlag) {
			GameState = MENU;
			*enterFlag = 0;
		}
		break;

	case GAMEWIN:
		sprintf(buffer, "YOU  WIN!!!");
		loadFancyString(9, 6, buffer, BLINK);

		if (*enterFlag) {
			GameState = MENU;
			*enterFlag = 0;
		}
		break;

	case TRACKING:

		loadInput(1);
		sprintf(buffer, "Object Tracking Demo");
		loadFancyString(5, 1, buffer, 0);
		if (lastGameTimeMs <= gameTimeMs - 1000) {
			lastGameTimeMs = gameTimeMs;
			sprintf(buffer, "fps %2u", frames);
			loadFancyString(12, 2, buffer, 0);
			frames = 0;
		}

		if (*enterFlag) {
			GameState = MENU;
			*enterFlag = 0;
		}
		break;

	default:
		break;
	}

}

/**
 * Loads the string into the OSD array at x,y
 */
void loadString(uint8_t x, uint8_t y, char *string) {
	while (*string) {
		Output_Data[y][x++] = remap_ascii_char(*string);
		string++;
	}
}

void loadStringPrgm(uint8_t x, uint8_t y, char *string) {
	while (pgm_read_byte(string) != 0x00) {
		Output_Data[y][x++] = remap_ascii_char((char) pgm_read_byte(string++));
	}
}

void loadFancyString(uint8_t x, uint8_t y, char *string, char attributes) {
	while (*string) {
		Output_Data[y][x] = remap_ascii_char(*string);
		Output_Attributes[y][x] = attributes;
		x++;
		string++;
	}
}

// Handy for when input data is noisy (as in bad reads from the adc)
// not needed with the PCB hardware
void filterInput() {
	//screen out bad values first
	for (uint8_t i = 0; i < ROWS; i++) {
		for (uint8_t j = 0; j < COLS; j++) {
			if (Input_Data[i][j] < NTSC_MIN || Input_Data[i][j] > NTSC_MAX) {
				//printf("Bad Value: %u at %u,%u\n", Input_Data[i][j], i, j);
				if (i == 0)
					Input_Data[i][j] = Input_Data[i + 1][j];
				else if (i == ROWS - 1)
					Input_Data[i][j] = Input_Data[i - 1][j];
				else
					Input_Data[i][j] = Input_Data[i - 1][j] / 2 + Input_Data[i + 1][j] / 2;
			}
		}
	}

	//the insides
	for (uint8_t i = 1; i < ROWS - 1; i++) {
		for (uint8_t j = 1; j < COLS - 1; j++) {
			Input_Data[i][j] = Input_Data[i][j] / 2 + Input_Data[i - 1][j] / 8 + Input_Data[i + 1][j] / 8
					+ Input_Data[i][j - 1] / 8 + Input_Data[i][j + 1] / 8;
		}
	}

	//four sides
	for (uint8_t i = 0; i < ROWS; i++) {
		Input_Data[i][0] = Input_Data[i][0] / 2 + Input_Data[i][0] / 8 + Input_Data[i - 1][0] / 8
				+ Input_Data[i + 1][0] / 8 + Input_Data[i][1] / 8;
		Input_Data[i][COLS - 1] = Input_Data[i][COLS - 1] / 2 + Input_Data[i][COLS - 1] / 8 + Input_Data[i - 1][COLS
				- 1] / 8 + Input_Data[i + 1][COLS - 1] / 8 + Input_Data[i][COLS - 2] / 8;
	}

	for (uint8_t j = 0; j < COLS; j++) {
		Input_Data[0][j] = Input_Data[0][j] / 2 + Input_Data[0][j] / 8 + Input_Data[1][j] / 8 + Input_Data[0][j - 1]
				/ 8 + Input_Data[0][j + 1] / 8;
		Input_Data[ROWS - 1][j] = Input_Data[ROWS - 1][j] / 2 + Input_Data[ROWS - 1][j] / 8 + Input_Data[ROWS - 2][j]
				/ 8 + Input_Data[ROWS - 1][j - 1] / 8 + Input_Data[ROWS - 1][j + 1] / 8;
	}

	//four corners
	Input_Data[0][0] = Input_Data[0][0] / 2 + Input_Data[1][0] / 4 + Input_Data[0][1] / 4;
	Input_Data[ROWS - 1][0] = Input_Data[ROWS - 1][0] / 2 + Input_Data[ROWS - 2][0] / 4 + Input_Data[ROWS - 1][1] / 4;
	Input_Data[0][COLS - 1] = Input_Data[0][COLS - 1] / 2 + Input_Data[1][COLS - 1] / 4 + Input_Data[0][COLS - 2] / 4;
	Input_Data[ROWS - 1][COLS - 1] = Input_Data[ROWS - 1][COLS - 1] / 2 + Input_Data[ROWS - 2][COLS - 1] / 4
			+ Input_Data[ROWS - 1][COLS - 2] / 4;
}

/**
 * dump input data to UART for debugging
 */
void debugDump() {
	printf("FRAME: \n\r");
	for (uint8_t i = 0; i < ROWS; i++) {
		for (uint8_t j = 0; j < COLS; j++) {
			printf("%3u ", Input_Data[i][j]);
		}
		printf("\n\r");
	}
}

/**
 * Loads NTSC input onto array for OSD display.
 * Wipes anything currently on the screen.
 */
void loadInput(char overWrite) {

	//load image data into osd display array
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0; j < COLS; j += 2) {
			//normal view
			topL = Input_Data[i][j] < Black_Level;
			midL = Input_Data[i + 1][j] < Black_Level;
			botL = Input_Data[i + 2][j] < Black_Level;
			topR = Input_Data[i][j + 1] < Black_Level;
			midR = Input_Data[i + 1][j + 1] < Black_Level;
			botR = Input_Data[i + 2][j + 1] < Black_Level;

			//noise view
			//			topL = Input_Data[i][j] < NTSC_MIN || Input_Data[i][j] > NTSC_MAX;
			//			midL = Input_Data[i + 1][j] < NTSC_MIN || Input_Data[i + 1][j] > NTSC_MAX;
			//			botL = Input_Data[i + 2][j] < NTSC_MIN || Input_Data[i + 2][j] > NTSC_MAX;
			//			topR = Input_Data[i][j + 1] < NTSC_MIN || Input_Data[i][j + 1] > NTSC_MAX;
			//			midR = Input_Data[i + 1][j + 1] < NTSC_MIN || Input_Data[i + 1][j + 1] > NTSC_MAX;
			//			botR = Input_Data[i + 2][j + 1] < NTSC_MIN || Input_Data[i + 2][j + 1] > NTSC_MAX;

			//store dots as binary number, each dot is a flag
			if (overWrite | topL | midL | botL | topR | midR | botR)
				Output_Data[i / 3][j / 2] = DOTS_BLANK + topL + 2 * topR + 4 * midL + 8 * midR + 16 * botL + 32 * botR;
		}
	}
}

void debounce(uint8_t button, uint8_t * buttonState, uint8_t * buttonFlag) {
	switch (*buttonState) {
	case NoPush:
		if (button)
			*buttonState = MaybePush;
		else
			*buttonState = NoPush;
		break;
	case MaybePush:
		if (button) {
			*buttonState = Pushed;
			*buttonFlag = 0x01;
		} else
			*buttonState = NoPush;
		break;
	case Pushed:
		if (button)
			*buttonState = Pushed;
		else
			*buttonState = MaybeNoPush;
		break;
	case MaybeNoPush:
		if (button)
			*buttonState = Pushed;
		else {
			*buttonState = NoPush;
		}
		break;
	}
}

//set output data to all blanks
void clearOutput() {
	for (uint8_t i = 0; i < 13; i++) {
		for (uint8_t j = 0; j < 30; j++) {
			Output_Data[i][j] = 0;
			Output_Attributes[i][j] = 0;
		}
	}
}

/**
 * Sends the OSD array to the OSD for display
 */
void display() {
	//center on TV
	spi_send_byte(0x02, 0b00100000); //HOS zero
	spi_send_byte(0x03, 0b00010000); //VOS zero

	// Setup Video Mode: NTSC + enable display immediately (VM0)
	spi_send_byte(0x00, 0b00001000);

	// VM1 default
	spi_send_byte(0x01, 0b01000111);

	// 8bit mode (DMM)
	spi_send_byte(0x04, 0b01000000);

	//actually show display
	for (uint8_t i = 0; i < 13; i++) {
		for (uint8_t j = 0; j < 30; j++) {
			write_char_xy(j, i, Output_Data[i][j]);
			write_char_att_xy(j, i, Output_Attributes[i][j]);
		}
	}

}

void init_sync(void) {

	CLR(HSYNC_DDR,HSYNC_DDR_PIN); //set as input
	CLR(VSYNC_DDR,VSYNC_DDR_PIN); //set as input

	CLR(EIMSK,VSYNC_ISR); //keep int1 off for now, hooked to osd's VSYNC
	CLR(EIMSK,HSYNC_ISR); //keep int0 off for now, hooked to osd's HSYNC

	//trigger on rising edges for HSYNC
	SET(EICRA,HSYNC_ISC0);
	SET(EICRA,HSYNC_ISC1);

	//trigger on low level for VSYNC
	CLR(EICRA,VSYNC_ISC0);
	CLR(EICRA,VSYNC_ISC1);
}

void init_MISC(void) {

	//set up the LED output port
	SET(LED_DDR,LED_DDR_PIN);

	//init timer 1 to generate 5ms timebase, isr then scales down to 100ms base
	OCR1A = TimePreScale - 1;
	TCCR1B = 0x08 | 0x02; //1/8th full speed; clear-on-match
	TCCR1A = 0x00; //turn off pwm and oc lines
	TIMSK1 = 0x02; //enable interrupt T1 cmp A

	//init the black level
	Black_Level = 0xA0;

	//set pb0 as input for debug button
	CLR(DDRB,DDB0);
}

void init_ADC(void) {
	//a7 as output zero, connected to ADC OutputEnable pin
	SET(DDRA,PA7);
	CLR(PORTA,PA7);

	//set up ADC input
	DDR_ADC_DATA = 0;

	//set up ADC CLK output register
	SET(DDR_ADC_CLK,ADC_CLK);

	// falling edge of CLK means sample N is taken and sample N-3 is readable
	// rising edge of CLK, can't read N-3 anymore
	// at next falling edge, N = N + 1
	// running this at a steady 1MHz does not cause a noticeable horizontal shift

	//TIMER2 for ADC CLK (D7 == OC2A)
	SET(TCCR2A,WGM01); // set clear timer on compare match CTC mode
	SET(TCCR2A,COM2A0); // toggle OC2A on compare match
	TCCR2B = 1; //no prescaling
	OCR2A = 1; // count 2 cycles to get 5Mhz out, 8 cycles for 1.25MHz
}

// Setup the the ADC.
void init_ADC2(void) {
	//init the A to D converter
	//channel zero/ left adj /EXTERNAL Aref
	//!!!CONNECT Aref jumper!!!!
	SET(ADMUX,ADLAR);
	SET(ADMUX,REFS0);
	CLR(ADMUX,REFS1);

	//enable ADC and set prescaler to 1/128*16MHz=125,000
	//and start a conversion
	SET(ADCSRA,ADEN);
	SET(ADCSRA,ADSC);
	SET(ADCSRA,ADPS2);
	SET(ADCSRA,ADPS1);
	SET(ADCSRA,ADPS0);

	//set A0 input
	CLR(DDRA,PA0);
}

void init_Buttons(void) {

	CLR(SW_DDR, SW1_DDR_PIN);
	CLR(SW_DDR, SW2_DDR_PIN);

}

