/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

#include <inttypes.h>
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#include "brick_breaker.h"
#include "lab5.h"
#include "macros.h"
#include "Levels.h"

// ========================== "PRIVATE" VARIABLE DECLARATIONS ==========================

#define LIFECOUNT 3

#define BORDER 1 //how much of the screen we are missing, in columns
uint8_t thisValue, lastValue; //used for time filtering the output

uint16_t currentLevelStartTimeMs;

int8_t levelCleared;

int8_t ballx, bally, ballvx, ballvy;

uint16_t last;
#define DT 50 //gameTimeMs IRL per timestep in game
// ========================== "PRIVATE" FUNCTION DECLARATIONS ==========================

void loadBricks();

void displayBar();
void updateBall();
void displayBricks();

// ========================== FUNCTION DEFINTIONS ==========================

void initBrickBreaker() {
	//setup gameplay stuff
	lives = LIFECOUNT;
	gameTimeMs = 0;
	last = gameTimeMs;
	currentLevel = 0;
	currentLevelStartTimeMs = gameTimeMs;
	levelCleared = 0;
	ballx = 6 * 2;
	bally = 7 * 3;
	ballvx = 1;
	ballvy = -1;
	clearOutput();
	loadBricks();
}

void DoBrickBreakerFrame() {

	if (lives < 1) {
		GameState = GAMEOVER;
		return;
	}

	//level advancing logic
	if (levelCleared /*|| *toggleFlag & CHEAT*/) {
		//*toggleFlag = 0;
		if (++currentLevel >= TOTAL_LEVELS) {
			GameState = GAMEWIN;
			return;
		} else {
			currentLevelStartTimeMs = gameTimeMs;
			levelCleared = 0;
			loadBricks();
		}

	}
	//actual frame
	clearOutput();

	//messaging logic
	if (gameTimeMs - currentLevelStartTimeMs <= 500) {
		sprintf(buffer, "LEVEL %02u", currentLevel);
		loadFancyString(10, 6, buffer, INVERT);
	}

	displayBricks();
	displayBar();

	updateBall(); //has to have bricks and bar onscreen

	displayBricks(); //update after ball hits them

	sprintf(buffer, " Lvl: %02u ", currentLevel);
	loadFancyString(1, 1, buffer, LBC);

	sprintf(buffer, " Lives: %1d ", lives);
	loadFancyString(19, 1, buffer, LBC);
}

//put person blocks in based on current and previous values
void displayBar() {
	//load image data into osd display array
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0; j < COLS; j += 2) {
			topL = Input_Data[i][j] < Black_Level;
			midL = Input_Data[i + 1][j] < Black_Level;
			botL = Input_Data[i + 2][j] < Black_Level;
			topR = Input_Data[i][j + 1] < Black_Level;
			midR = Input_Data[i + 1][j + 1] < Black_Level;
			botR = Input_Data[i + 2][j + 1] < Black_Level;

			//store dots as binary number, each dot is a flag
			thisValue = (topL + midL + botL + topR + midR + botR > 3);
			lastValue = Output_Data[i / 3][j / 2];

			//filter over time
			if (lastValue == BLANK) {
				Output_Data[i / 3][j / 2] = (thisValue) ? PERSON_BLOCK_IN + 8 : BLANK;
			} else if (lastValue >= PERSON_BLOCK_IN && lastValue < PERSON_BLOCK_IN + 16) {
				Output_Data[i / 3][j / 2] = (thisValue) ? MIN(PERSON_BLOCK_IN + 15, lastValue+3)
						: MAX(PERSON_BLOCK_IN, lastValue-3);
			}
		}
	}
}

//register a hit on a brick to damage it
void hitBrick(int8_t y, int8_t x) {
	for (uint8_t b = 0; b < BRICK_COUNT; ++b) {
		if (Bricks[b][0] == y && Bricks[b][1] == x)
			Bricks[b][2]--;
	}
}

//query what object is at location x,y
uint8_t lookAt(int8_t x, int8_t y) {

	//screen values
	if (Output_Data[y / (3)][x / (2)] >= (PERSON_BLOCK_IN) && Output_Data[y / (3)][x / (2)] < (PERSON_BLOCK_IN + 8))
		return BLANK;
	else
		switch (Output_Data[y / (3)][x / (2)]) {
		case BRICK:
		case BRICK_CRACKED:
		case BRICK_REALLY_CRACKED:
			hitBrick(y / 3, (x / 2));
			break;
		case BRICK + 1:
		case BRICK_REALLY_CRACKED + 1:
		case BRICK_CRACKED + 1:
			hitBrick(y / 3, (x / 2) - 1);
			break;
		case BRICK + 2:
		case BRICK_CRACKED + 2:
		case BRICK_REALLY_CRACKED + 2:
			hitBrick(y / 3, (x / 2) - 2);
			break;
		default:

			break;
		}

	return Output_Data[y / (3)][x / (2)];
}

// call after display person and display bricks
// it assumes those blocks are already in the output buffer
void updateBall() {
	if (gameTimeMs - last >= DT) {

		last = gameTimeMs;
		if (bally + ballvy >= ROWS) {
			lives--;
			ballx = 6 * 2;
			bally = 7 * 3;
			ballvx = 1;
			ballvy = -1;
		} else {

			//ball travels in 39x60 (y by x) world
			//collide first, check x and y separately (use x result in y check)
			if (lookAt(ballx + ballvx, bally) != BLANK) {
				ballvx *= -1;
			}
			if (lookAt(ballx + ballvx, bally + ballvy) != BLANK) {
				ballvy *= -1;
			}

			//then advance ball
			ballx += ballvx;
			bally += ballvy;
		}
	}

	//draw ball in 13 by 30 world, at a sub-character location
	char r = (bally) % (3);
	char c = (ballx) % (2);

	//balls stored in a row 1 thru 6
	Output_Data[bally / (3)][ballx / (2)] = BALL_BLANK + 1 + (2 * r + c);

}

//from progmem to RAM
void loadBricks() {
	for (uint8_t b = 0; b < BRICK_COUNT; b++) {
		Bricks[b][0] = (uint8_t) pgm_read_byte(&Levels[currentLevel][b][0]);
		Bricks[b][1] = (uint8_t) pgm_read_byte(&Levels[currentLevel][b][1]);
		Bricks[b][2] = (uint8_t) pgm_read_byte(&Levels[currentLevel][b][2]);
	}
}

//show all the current bricks onscreen, as well as game boundary walls
void displayBricks() {
	uint8_t bricksLeft = 0;
	for (uint8_t b = 0; b < BRICK_COUNT; ++b) {
		switch (Bricks[b][2]) {
		case 3:
			bricksLeft++;
			Output_Data[Bricks[b][0]][Bricks[b][1]] = BRICK;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 1] = BRICK + 1;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 2] = BRICK + 2;
			break;
		case 2:
			bricksLeft++;
			Output_Data[Bricks[b][0]][Bricks[b][1]] = BRICK_CRACKED;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 1] = BRICK_CRACKED + 1;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 2] = BRICK_CRACKED + 2;
			break;
		case 1:
			bricksLeft++;
			Output_Data[Bricks[b][0]][Bricks[b][1]] = BRICK_REALLY_CRACKED;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 1] = BRICK_REALLY_CRACKED + 1;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 2] = BRICK_REALLY_CRACKED + 2;
			break;
		case 0:
			//brick is dead
			break;
		default:
			//brick is invulnerable
			Output_Data[Bricks[b][0]][Bricks[b][1]] = BRICK_UNBREAKABLE;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 1] = BRICK_UNBREAKABLE + 1;
			Output_Data[Bricks[b][0]][Bricks[b][1] + 2] = BRICK_UNBREAKABLE + 2;
			break;
			break;
		}
	}

	if (bricksLeft == 0) {
		levelCleared = 1;
	}

	//side walls
	for (uint8_t i = 0; i < ROWS / 3; i++) {
		Output_Data[i][0] = WALL_BLOCK;
		Output_Data[i][1] = WALL_BLOCK;
		Output_Data[i][2] = WALL_BLOCK;
		Output_Data[i][3] = WALL_BLOCK;
		Output_Data[i][COLS / 2 - 4] = WALL_BLOCK;
		Output_Data[i][COLS / 2 - 3] = WALL_BLOCK;
		Output_Data[i][COLS / 2 - 2] = WALL_BLOCK;
		Output_Data[i][COLS / 2 - 1] = WALL_BLOCK;
	}

	//top wall
	for (uint8_t j = 0; j < COLS / 2; j++) {
		Output_Data[0][j] = WALL_BLOCK;
	}
}
