/*****************************
 * ECE 4760: Final Project   *
 * Human Tetris              *
 * Video Arcade Edition      *
 *                           *
 * Adam Papamarcos (aip23)   *
 * Kerran Flanagan (kaf42)   *
 * Cornell University        *
 * Spring 2010               *
 *****************************/

#include <inttypes.h>
#include <avr/io.h>
#include <string.h>
#include <stdio.h>
#include <avr/pgmspace.h>

#include "whack_a_mole.h"
#include "lab5.h"

#include "macros.h"

uint8_t thisValue, lastValue;
uint8_t molesHit;
uint16_t lastMoleTime;

// ========================== "PRIVATE" FUNCTION DECLARATIONS ==========================

void displayBody();
void displayMole(uint8_t x, uint8_t y, uint8_t write);
inline uint8_t isMole(uint8_t val);
inline void hitMole(uint8_t i, uint8_t j);

// ========================== FUNCTION DEFINTIONS ==========================

void initMole() {
	//setup gameplay stuff
	gameTimeMs = 0;
	currentLevel = 0;
	endTimeMs = gameTimeMs + 15000;
	molesHit = 0;
	lastMoleTime = gameTimeMs;
	clearOutput();
}

#define MOLES 18
uint8_t moleSpots[MOLES][2] = { { 3, 2 }, { 7, 2 }, { 11, 2 }, { 15, 2 }, { 19, 2 }, { 23, 2 }, { 3, 5 }, { 7, 5 },
								 { 11, 5 }, { 15, 5 }, { 19, 5 }, { 23, 5 }, { 3, 8 }, { 7, 8 }, { 11, 8 }, { 15, 8 },
								 { 19, 8 }, { 23, 8 } };
uint8_t midx;

void DoMoleFrame() {
	//level advancing logic
	if (endTimeMs <= gameTimeMs) {
		GameState = GAMEWIN;
		return;
	}

	if (lastMoleTime + 1000 <= gameTimeMs) {
		midx = (gameTimeMs * 7) % MOLES;
		displayMole(moleSpots[midx][0], moleSpots[midx][1], 1);
		lastMoleTime = gameTimeMs;
	}

	if (lastMoleTime + 400 <= gameTimeMs && lastMoleTime + 600 >= gameTimeMs) {
		//make moles disappear (randomly blank out spots)
		midx = (gameTimeMs * 11) % MOLES;
		displayMole(moleSpots[midx][0], moleSpots[midx][1], 0);
	}

	//messaging logic
	sprintf(buffer, " Moles Hit: %2u ", molesHit / 6);
	loadFancyString(1, 1, buffer, LBC);

	sprintf(buffer, " Time: %02u ", (endTimeMs - gameTimeMs + 500) / 1000);
	loadFancyString(19, 1, buffer, LBC);

	//actual frame
	displayBody();
}

//put person blocks in based on previous value
//remember shape is only written to output once per shape
void displayBody() {
	//load image data into osd display array
	for (uint8_t i = 0; i < ROWS; i += 3) {
		for (uint8_t j = 0; j < COLS; j += 2) {
			topL = Input_Data[i][j] < Black_Level;
			midL = Input_Data[i + 1][j] < Black_Level;
			botL = Input_Data[i + 2][j] < Black_Level;
			topR = Input_Data[i][j + 1] < Black_Level;
			midR = Input_Data[i + 1][j + 1] < Black_Level;
			botR = Input_Data[i + 2][j + 1] < Black_Level;

			//store dots as binary number, each dot is a flag
			thisValue = (topL + midL + botL + topR + midR + botR > 3);
			lastValue = Output_Data[i / 3][j / 2];

			//filter over time
			if (lastValue == BLANK) {
				Output_Data[i / 3][j / 2] = (thisValue) ? PERSON_BLOCK_IN + 8 : BLANK;
			} else if (lastValue >= PERSON_BLOCK_IN && lastValue < PERSON_BLOCK_IN + 16) {
				Output_Data[i / 3][j / 2] = (thisValue) ? MIN(PERSON_BLOCK_IN + 15, lastValue+2)
						: MAX(PERSON_BLOCK_IN, lastValue-2);
			} else if (isMole(lastValue) && thisValue) {
				Output_Data[i / 3][j / 2] = PERSON_BLOCK_OUT + 8;
				hitMole(i, j);
			} else if (lastValue == PERSON_BLOCK_OUT + 8) {
				Output_Data[i / 3][j / 2] = (thisValue) ? PERSON_BLOCK_IN + 8 : BLANK;
			}

		}
	}
}

inline void hitMole(uint8_t i, uint8_t j) {
	molesHit++;
}

inline uint8_t isMole(uint8_t val) {
	return val == MOLE_BLOCK_TOP || val == MOLE_BLOCK_TOP + 1 || val == MOLE_BLOCK_TOP + 2 || val == MOLE_BLOCK_BOT
			|| val == MOLE_BLOCK_BOT + 1 || val == MOLE_BLOCK_BOT + 2;
}

void displayMole(uint8_t x, uint8_t y, uint8_t write) {
	Output_Data[y][x] = (write) ? MOLE_BLOCK_TOP : BLANK;
	Output_Data[y][x + 1] = (write) ? MOLE_BLOCK_TOP + 1 : BLANK;
	Output_Data[y][x + 2] = (write) ? MOLE_BLOCK_TOP + 2 : BLANK;
	Output_Data[y + 1][x] = (write) ? MOLE_BLOCK_BOT : BLANK;
	Output_Data[y + 1][x + 1] = (write) ? MOLE_BLOCK_BOT + 1 : BLANK;
	Output_Data[y + 1][x + 2] = (write) ? MOLE_BLOCK_BOT + 2 : BLANK;
}
